/*
 * Game & Watch Volleyball, an "Arcade Volleyball" clone with reworked Game & Watch
 * sprites based upon an early version of "Volleyball", originally coded by Hugo Ruscitti
 * You may check Hugo's web site for more info: http://www.loosersjuegos.com.ar
 * Game & Watch Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Game & Watch Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <SDL.h>
#include <stdlib.h>
#include "juego.h"
#include "sonido.h"


SDL_Surface * fondoA;
SDL_Surface * fondoB;
SDL_Event evento;

int juego_iniciar (Juego * juego)
{
	if (juego_cargar_fondo (juego))
		return 1;

	if (juego_iniciar_procesos (juego))
		return 1;
	
	SDL_BlitSurface (juego->fondo, NULL, juego->screen, NULL);
	SDL_Flip (juego->screen);
	abrirAudio();
	dosJugadores=0;
	return 0;
}

int juego_cargar_fondo (Juego * juego)
{
	fondoA = cargar_imagen ("data/fondoA.bmp", 0);
	fondoB = cargar_imagen ("data/fondoB.bmp", 0);
			
	if (dosJugadores==0) juego->fondo = fondoA;
	else juego->fondo = fondoB;
		
	if (juego->fondo == NULL)
	{
		return 1;
	}

	return 0;
}


void juego_cambiar_fondo (Juego * juego)
{
	if (dosJugadores==0) juego->fondo = fondoA;
	else juego->fondo = fondoB;
	reproducirSonido(SND_ANDAR);
}


int juego_iniciar_procesos (Juego * juego)
{
	if (pelota_iniciar (& juego->pelota))
		return 1;

	if (dirty_iniciar (& juego->dirty))
		return 1;

	if (personaje_crear (& juego->personaje_1, 1))
		return 1;

	if (personaje_crear (& juego->personaje_2, 2))
		return 1;

	if (puntaje_crear (& juego->puntaje))
		return 1;
	
	return 0;
}


void juego_ciclo_logico (Juego * juego)
{

	Uint8 teclas[6];
	teclas[0] = SDL_JoystickGetButton(juego->joy, GP2X_LEFT);
	teclas[1] = SDL_JoystickGetButton(juego->joy, GP2X_RIGHT);
	if (dosJugadores != 1) teclas[2] = SDL_JoystickGetButton(juego->joy, GP2X_X);
	else teclas[2] = SDL_JoystickGetButton(juego->joy, GP2X_L);
	
	if (dosJugadores==1)
	{
		teclas[3] = SDL_JoystickGetButton(juego->joy, GP2X_A);
		teclas[4] = SDL_JoystickGetButton(juego->joy, GP2X_B);
		teclas[5] = SDL_JoystickGetButton(juego->joy, GP2X_R);
	}	
	else /* Control automtico del segundo personaje. Sencillo, pero funciona */
	{	
		if (juego->pelota->x >= 320) 
		{
			if(juego->pelota->x + 25 < juego->personaje_2->x)
			{
				teclas[3] = 1;
				teclas[4] = 0;
			}
			else if((juego->pelota->x + 25 > juego->personaje_2->x)
					 && (juego->pelota->x + 10 < juego->personaje_2->x)) 		
			{
				teclas[3] = 0;
				teclas[4] = 0;
			}
			else
			{
				teclas[3] = 0;
				teclas[4] = 1;
			}

			if(juego->pelota->y < 300 && juego->pelota->y > 260) // Es 260 de altura para poder sacar
			{
				teclas[5] = 1;
			}
			else
				teclas[5] = 0;
		}
		else if (juego->personaje_2->x < 470) // Centramos al jugador 2 si la bola no est en su campo
			{
				teclas[3] = 0;
				teclas[4] = 1;
				teclas[5] = 0;
			}
			else if (juego->personaje_2->x > 490)		
			{
				teclas[3] = 1;
				teclas[4] = 0;
				teclas[5] = 0;
			}
			else
			{
				teclas[3] = 0;
				teclas[4] = 0;
				teclas[5] = 0;
			}		
		}

	personaje_actualizar (juego->personaje_1, teclas);
	personaje_actualizar (juego->personaje_2, teclas);

	pelota_actualizar (juego->pelota, juego->personaje_1, \
			juego->personaje_2, juego->puntaje);
	
	if (cambioFondo == 1) 
	{
		cambioFondo = 0;
		juego_dosJugadores(juego);
	}		
}


void juego_ciclo_grafico (Juego * juego)
{
	dirty_limpiar (juego->dirty, juego->screen, juego->fondo);
	
	personaje_imprimir (juego->personaje_1, juego->dirty, juego->screen);
	personaje_imprimir (juego->personaje_2, juego->dirty, juego->screen);
	pelota_imprimir (juego->pelota, juego->dirty , juego->screen);
	
	puntaje_imprimir (juego->puntaje, juego->dirty, juego->screen);

	dirty_actualizar (juego->dirty, juego->screen);
}


void juego_dosJugadores(Juego* juego)
{
	if (dosJugadores==0) dosJugadores=1;
	else dosJugadores=0; 
	juego_cambiar_fondo(juego);
	juego_iniciar_procesos(juego);
}


void juego_terminar (Juego * juego)
{
	pelota_terminar (juego->pelota);
	personaje_terminar (juego->personaje_1);
	personaje_terminar (juego->personaje_2);
	dirty_terminar (juego->dirty);
	puntaje_terminar (juego->puntaje);
	SDL_FillRect(juego->screen,NULL,0);
	SDL_JoystickClose(juego->joy);
	SDL_FreeSurface (juego->fondo);
	cerrarAudio();
}

void juego_cambioFlag()
{
	cambioFondo = 1;
}

