/*
 * Game & Watch Volleyball, an "Arcade Volleyball" clone with reworked Game & Watch
 * sprites based upon an early version of "Volleyball", originally coded by Hugo Ruscitti
 * You may check Hugo's web site for more info: http://www.loosersjuegos.com.ar
 * Game & Watch Volleyball is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Game & Watch Volleyball is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "util.h"
#include "sonido.h"
#include "juego.h"

/*
 * retorna: 1 en caso de error
 */
int iniciar_sdl (SDL_Surface ** screen, SDL_Joystick** joy)
{
	
	if (SDL_Init (SDL_INIT_VIDEO | SDL_INIT_JOYSTICK))
	{
		return 1;
	}

	//Inicializacin del "stick" de GP2x
	*joy = SDL_JoystickOpen(0);

	* screen = SDL_SetVideoMode (640, 480, 16, SDL_HWSURFACE | SDL_DOUBLEBUF);

	SDL_ShowCursor(0);

	if (* screen == NULL) return 1;
	SDL_WM_SetCaption (TITULO, NULL);
	return 0;
}

/*
 * Devuelve 0 si el programa va a salir al men, 1 en otro caso.
 */
int procesar_eventos ()
{
	static SDL_Event event;

	while (SDL_PollEvent (&event))
	{
		if (event.type == SDL_QUIT)
			return 0;

		if (event.type == SDL_JOYBUTTONDOWN)
		{
			switch (event.jbutton.button)
			{
				case GP2X_START:
					return 0;
					break;

				default:
					break;
			}
		}
		
		if (event.type == SDL_JOYBUTTONUP)
		{
			switch (event.jbutton.button)
			{
				case GP2X_SELECT:
          		{
          			reproducirSonido(SND_ANDAR);
          			// Se ha soltado el botn Y
					juego_cambioFlag();
          			break;
          		}
          		
          		case GP2X_VOLUP:
          		{
          			aumentarVolumen();
          			break;
          		}
          		
          		case GP2X_VOLDOWN:
          		{
          			reducirVolumen();
          			break;
          		}
          		
			}
		} 
	}

	return 1;
}

SDL_Surface * cargar_imagen (char * ruta, int con_color_clave)
{
	SDL_Surface * tmp;
	SDL_Surface * tmp2;

	tmp = SDL_LoadBMP (ruta);

	if (tmp == NULL)
	{
		return NULL;
	}
	
	/* Optimiza la imagen */
	tmp2 = SDL_DisplayFormat (tmp);

	if (tmp2 == NULL){}
	else
	{
		SDL_FreeSurface (tmp);
		tmp = tmp2;
	}

	
	/* Aplica la transparencia (COLOR_KEY) */
	if (con_color_clave)
	{
		SDL_SetColorKey (tmp, SDL_SRCCOLORKEY, \
				SDL_MapRGB (tmp->format, 255, 0, 255));
	}

	return tmp;
}

int colisionan (int x0, int y0, float x1, float y1, int suma_radios)
{
	int x = abs (x0 - x1);
	int y = abs (y0 - y1);
	double distancia = sqrt (x * x + y * y);
	
	if (suma_radios > distancia)
		return 1;
		
	return 0;
}

void imprimir_grilla (SDL_Surface * src, int cuadro, SDL_Surface * dst, SDL_Rect * rect, int columnas)
{
	int w = src->w / columnas;
	SDL_Rect fuente = {cuadro * w, 0, w, src->h};

	SDL_BlitSurface (src, & fuente, dst, rect);
}
